import javax.sound.midi.*;
import java.io.*;
import java.util.*;

public class Example
    {
    // A helper class which outputs the name of the MIDI device properly so
    // JComboBoxes in JOptionPane can display them nicely.
    static class MidiDeviceName
        {
        public MidiDevice device;
        public MidiDeviceName(MidiDevice d) { device = d; }
        public String toString() 
        	{ 
        	String desc = device.getDeviceInfo().getDescription().trim();
        	
        	// All CoreMIDI4J names begin with "CoreMIDI4J - "
        	String name = device.getDeviceInfo().getName().substring(13).trim();

        	if (name.equals(""))
	        	return desc; 
			else 
	        	return desc + ": " + name; 
        	}
        }
    
    // Synchronize the device access.  Not that it'll make much difference
    // as I'm trying to make sure everything is accessed from the Swing Event Thread
    Object lock = new Object[0];
        
    // Our Keyboard
    MidiDevice key = null;
    MidiDeviceName keyn = null;
    Transmitter keyr = null;
    public static final int KEYCHANNEL_NONE = 0;
    int keyChannel = KEYCHANNEL_NONE;
    
    // Our input device
    MidiDevice in = null;   
    MidiDeviceName inn = null;
    Transmitter inr = null;
        
    // Our output device
    MidiDevice out = null;
    MidiDeviceName outn = null;
    Receiver outr = null;
    int sendChannel = 1;
        
    /** Attaches the synth to receive sysex provided by the transmitter.  To do
        this, the transmitter must actually exist (not be null). */
    public void attachToTransmitter()
        {
        if (inr != null) 
        	inr.setReceiver(new Receiver()
            {
            public void close()
                {
                // this doesn't matter since we're not returning the Receiver, but whatever
                synchronized(lock) 
                    { 
                    if (inr != null) 
                        inr.close(); 
                    }
                inr = null;
                }
                                
            public void send(MidiMessage message, long timeStamp)
                {
                // nothing for now
                }
            });

        if (keyr != null) 
        	keyr.setReceiver(new Receiver()
            {
            public void close()
                {
                // this doesn't matter since we're not returning the Receiver, but whatever
                synchronized(lock) 
                    { 
                    if (keyr != null) 
                        keyr.close(); 
                    }
                keyr = null;
                }
                                
            public void send(MidiMessage message, long timeStamp)
                {
                // nothing for now
        		}
        	});
        }



	public void disconnectMIDI()
		{
		synchronized(lock)
			{
			if (inr != null)
				inr.close();
				
			if (in != null)
				{
				in.close();
				}
				
			if (outr != null)
				{
				outr.close();
				}

			if (out != null)
				{
				out.close();
				}
				
			if (keyr != null)
				{
				keyr.close(); 
				}

			if (key != null)
				{
				key.close();
				}
				
			inr = null;
			in = null;
			inn = null;
			outr = null;
			out = null;
			outn = null;
		    }
		}


    public static final int FAILED = -1;
    public static final int CANCELLED = 0;
    public static final int SUCCEEDED = 1;
    
	public int gatherMidiDevices()
		{
        synchronized(lock)
            {
            // Obtain information about all the installed synthesizers.
            MidiDevice.Info[] midi = uk.co.xfactorylibrarians.coremidi4j.CoreMidiDeviceProvider.getMidiDeviceInfo();
            ArrayList transdevices = new ArrayList();
            for(int i = 0; i < midi.length; i++)
                {
                try
                    {
                    MidiDevice d = MidiSystem.getMidiDevice(midi[i]);
                    // get rid of java devices
                    if (d instanceof javax.sound.midi.Sequencer ||
                    	d instanceof javax.sound.midi.Synthesizer)
                    		continue;
                    if (d.getMaxTransmitters() != 0)
                        {
                        transdevices.add(new MidiDeviceName(d));
                        }
                    }
                catch(Exception e) { }
                }

            ArrayList recvdevices = new ArrayList();
            for(int i = 0; i < midi.length; i++)
                {
                try
                    {
                    MidiDevice d = MidiSystem.getMidiDevice(midi[i]);
                    // get rid of java devices
                    if (d instanceof javax.sound.midi.Sequencer ||
                   		d instanceof javax.sound.midi.Synthesizer)
                    		continue;
                    if (d.getMaxReceivers() != 0)
                        {
                        recvdevices.add(new MidiDeviceName(d));
                        }
                    }
                catch(Exception e) { }
                }

            ArrayList keydevices = new ArrayList();
            keydevices.add("None");
            for(int i = 0; i < midi.length; i++)
                {
                try
                    {
                    MidiDevice d = MidiSystem.getMidiDevice(midi[i]);
                    // get rid of java devices
                    if (d instanceof javax.sound.midi.Sequencer ||
                   		d instanceof javax.sound.midi.Synthesizer)
                    		continue;
                    if (d.getMaxTransmitters() != 0)
                        {
                        keydevices.add(new MidiDeviceName(d));
                        }
                    }
                catch(Exception e) { e.printStackTrace(); }
                }
			
				Transmitter transmitter;
				Object choice = transdevices.get(0);
				try { ((MidiDeviceName)choice).device.open(); }
				catch (Exception e) { e.printStackTrace(); return FAILED; }
				try { transmitter = ((MidiDeviceName)choice).device.getTransmitter(); }
				catch (Exception e) 
					{ 
					e.printStackTrace(); 
					((MidiDeviceName)choice).device.close(); 
					return FAILED; 
					}

				if (inr != null)
					inr.close(); 
				if (in != null)
					in.close();
				inn = (MidiDeviceName) choice;
				in = ((MidiDeviceName)choice).device;
				inr = transmitter;

				// receiver

				Receiver receiver;
				choice = recvdevices.get(0);
				try { ((MidiDeviceName)choice).device.open(); }
				catch (Exception e) 
					{ 
					e.printStackTrace(); 
					in.close();
					inr.close();
					return FAILED; 
					}
				try { receiver = ((MidiDeviceName)choice).device.getReceiver(); }
				catch (Exception e) 
					{ 
					e.printStackTrace(); 
					((MidiDeviceName)choice).device.close();
					in.close();
					inr.close();
					return FAILED; 
					}

				if (outr != null)
					outr.close();
				if (out != null)
					out.close();
				outn = (MidiDeviceName) choice;
				out = ((MidiDeviceName)choice).device;
				outr = receiver;
				
				// keyboard
				
				Transmitter keybd;
				choice = keydevices.get(0);
				if (!(choice instanceof String)) // it's "None"
					{
					try { ((MidiDeviceName)choice).device.open(); }
					catch (Exception e) 
						{ 
						e.printStackTrace(); 
						return SUCCEEDED;
						}
					try { transmitter = ((MidiDeviceName)choice).device.getTransmitter(); }
					catch (Exception e) 
						{ 
						e.printStackTrace(); 
						return SUCCEEDED; 
						}

					if (keyr != null)
						keyr.close(); 
					if (key != null)
						key.close();
					keyn = (MidiDeviceName) choice;
					key = ((MidiDeviceName)choice).device;
					keyr = transmitter;
					}
				else
					{
					if (keyr != null)
						keyr.close(); 
					if (key != null)
						key.close();
					keyn = null;
					}
					
				keyChannel = 1;
				sendChannel = 1;
				return SUCCEEDED;
				}
			}
			
		public static void main(String[] args)
			{
			Example example = new Example();
			while(true)
				{
				example.gatherMidiDevices();
				example.attachToTransmitter();

				example.gatherMidiDevices();
				example.attachToTransmitter();

				example.gatherMidiDevices();
				example.attachToTransmitter();
				
				example.disconnectMIDI();
				}
			}
		}
